package mastermind.client;

/**
 * Mastermind GUI.
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-11.v01   ReMa    init version
 * 2012-05-13.v01   ReMa    div Neuerungen (MouseListener, Results, etc.)
 * 2012-05-14.v01   ReMa    Struktur überarbeitet GUI
 * 2012-05-15.v01   ReMa    vorbereitung für History
 * 2012-05-16.v01   ReMa    Engine eingebunden
 * 2012-05-19.v01   FeRo    Load / Save hinzugefügt
 * 2012-05-25.v01   ReMa    Solver Gui SetHiddenPeg
 * -----------------------------------------------------------------------------
 * @author Marco Reichmuth <marco.reichmuth@stud.hslu.ch>
 * @version 2012-05-19.v01
 */
import java.awt.Cursor;
import java.awt.event.*;
import java.io.File;
import java.util.Random;
import java.util.concurrent.TimeUnit;
import javax.swing.*;
import mastermind.logic.SettingsHandler;
import mastermind.logic.GuessHandler;
import mastermind.logic.RandomGuess;
import mastermind.engine.MastermindEngine;
import mastermind.model.Guess;
import mastermind.model.Result;
import mastermind.model.Settings;
import mastermind.solver.SolverLogik;

/**
 * Grafische Oberfläche um Spiel  zu bedienen
 * @author Marco Reichmuth <marco.reichmuth@stud.hslu.ch>
 */
public class ClientGUI extends javax.swing.JFrame implements IClient
{
    // Mastermind Engine

    private MastermindEngine mEngine;
    // Panel
    private JPanel gui_panel; // Hauptpanel
    private JPanel[] arrJPan; // Guesspanels
    private JLabel gui_status; // statusbar
    private JLabel gui_solver_status; // status label
    // MenuBar and MenuItems
    private JMenuBar gui_menubar; // Menubar
    private JMenu gui_menu_game; // Menu Game
    private JMenu gui_menu_help; // Menu Help
    private JMenu gui_menu_cheat; // Menu Cheat
    private JMenuItem gui_mi_new; // Menuitem new
    private JMenuItem gui_mi_newSolver; // Menuitem new Solver
    private JMenuItem gui_mi_open; // Menuitem open
    private JMenuItem gui_mi_save; // Menuitem save
    private JMenuItem gui_mi_settings; // Menuitem settings
    private JMenuItem gui_mi_exit; // Menuitem Exit
    private JMenuItem gui_mi_help; // Menuitem Help
    private JMenuItem gui_mi_about; // Menuitem About
    private JMenuItem gui_mi_showHidden; // Menuitem showHidden
    private JMenuItem gui_mi_showHiddenPeg; // Menuitem showHiddenPeg
    // Icons
    private ImageIcon[] arrGFXColor = new ImageIcon[Settings.ANZ_COLOR_MAX + 3];
    private static final int ID_GFX_EMPTY = 0;
    private static final int ID_GFX_EMPTY_DISABLED = 10;
    private static final int ID_GFX_QUESTION = 9;
    private ImageIcon[] arrGFXResult = new ImageIcon[3];
    private static final int ID_GFX_RESULT_BLACK = 1;
    private static final int ID_GFX_RESULT_WHITE = 2;
    private ImageIcon icon_check;
    private ImageIcon icon_warning;
    private ImageIcon icon_ok;
    private ImageIcon icon_disabled;
    private ImageIcon icon_info;
    // Arrays
    private JLabel arrIcon_guess[][];
    private JLabel arrIcon_result[][];
    private JLabel arrIcon_pattern[];
    //Buttons
    private JButton arrJButton[];
    // Attribute für Anzahl Steckplätze etc.
    private int nbrOfIcons;
    private int nbrOfRows;
    private int nbrOfPegs;
    // Arrays für Spielsteine
    private int[][] arrVal_guess; // Array für guess
    private int[][] arrVal_result; // Array für result
    private int[] arrVal_pattern; // Array für pattern
    private int nbrOfFilledRows;
    // Hiddenpattern anpassbar
    private boolean chg_hidden;
    // Abstände für die Grafischen Elemente
    // pattern Elemente
    private static final int gapx_pattern_b = 15;
    private static final int gapy_pattern_b = 10;
    private static final int sizex_pattern = 40;
    private static final int sizey_pattern = 40;
    private static final int gapx_pattern = sizex_pattern + 5; // Abstand zwischen guess-Labels
    // guess Elemente
    private static final int gapx_guess_b = gapx_pattern_b - 5; // Abstand von Rand für guess_labels
    private static final int gapy_guess_b = 5;
    private static final int sizex_guess = 40;
    private static final int sizey_guess = 40;
    private static final int gapx_guess = sizex_guess + 5; // Abstand zwischen guess-Labels
    private static final int gapy_guess = sizey_guess + 10; // Abstand zwischen guess-Labels
    // button Elemente
    private int gapx_button_b; // Abstand von Rand für button
    private static final int sizex_button = 40;
    private static final int sizey_button = 40;
    private static final int gapx_button = 50; // Abstand zwischen buttons
    private static final int gapy_button_b = gapy_guess_b; // Abstand von Rand für button
    // result Elemente
    private int gapx_result_b; // Abstand von Rand für result_labels
    private static final int gapy_result_b = gapy_guess_b; // Abstand von Rand für result_labels
    private static final int sizex_result = 20;
    private static final int sizey_result = 20;
    private static final int gapx_result = 25; // Abstand zwischen result-Labels
    private static final int gapy_result = gapy_guess / 2; // Abstand zwischen result-Labels
    private static final int gapy_result_o1 = -2; // Offset für 1. Reihe
    private static final int gapy_result_o2 = -4; // Offset für 2. Reihe
    // PanelElemente
    private static final int gapx_panel_b = gapx_pattern_b - 10;
    private static final int gapy_panel_b = gapy_pattern_b + sizey_pattern + 10;
    private static final int gapy_panel = gapy_guess;
    private int sizex_panel;
    private static final int sizey_panel = gapy_guess;
    // Status grösse
    private static final int sizey_status = 50;
    private int sizex_status;
    private static final int gapx_status_b = gapx_pattern_b;
    private int gapy_status_b;
    // GUI Grösse
    private int sizex_gui;
    private int sizey_gui;
    // Listener
    private MouseListener ml;
    private ActionListener al;
    private MouseWheelListener mwl;
    // Dialog
    JDialog myDialog;

    /**
     * leerer Default Konstruktor
     */
    public ClientGUI()
    {
        // nothing so far...
    }

    /**
     * init & run GUI
     * @param mEngine Mastermind Engine
     * @param args CMD-Args
     */
    public void runClient(MastermindEngine mEngine, String[] args)
    {
        // Set Mastermind Engine
        this.mEngine = mEngine;

        // Generate Random Pattern
        System.out.println("Secret Code wird erzeugt...");
        mEngine.setHiddenPattern(RandomGuess.genRandomGuess(), true);

        // Play
        doPlayMastermind();
        // in der mitte vom bildschirm anzeigen
        this.setLocationRelativeTo(null);
    }

    /**
     * Play Mastermind
     */
    private void doPlayMastermind()
    {
        // Play
        initSettings();
        loadDesign(SettingsHandler.getInstance().getSettings().getDesign()); // load icon-set
        initGUI();

        this.setVisible(true);
        //System.out.println("Code: " + mEngine.getHiddenPattern());
    }

    /**
     * Init Mastermind Settings
     */
    private void initSettings()
    {
        //System.out.println("initSettings...");
        nbrOfRows = SettingsHandler.getInstance().getSettings().getNumberOfRows();
        nbrOfPegs = SettingsHandler.getInstance().getSettings().getNumberOfPegs();
        nbrOfIcons = SettingsHandler.getInstance().getSettings().getNumberOfColors();
        nbrOfFilledRows = mEngine.getNumberOfGuesses();
    }

    /**
     * Load Design
     * @param design ID des Design
     */
    private void loadDesign(int design)
    {
        //System.out.println("loadDesign: " + design + "...");
        // button und result elemente
        gapx_button_b = nbrOfPegs * gapx_guess + gapx_guess_b + 10; // Abstand von Rand für button
        gapx_result_b = gapx_button + gapx_button_b + 10; // Abstand von Rand für result_labels
        // Panel Grösse
        sizex_panel = gapx_result_b + (nbrOfPegs + 1) / 2 * gapx_result + 5;
        gapy_status_b = (nbrOfRows) * gapy_panel + gapy_panel_b - 5;
        // Status Grösse
        sizex_status = sizex_panel;
        // GUI Grösse
        sizex_gui = sizex_panel + 15;
        sizey_gui = gapy_panel_b + (gapy_guess * (nbrOfRows + 1)) + sizey_status - 5;

        // Icons-Pool
        switch (design) {
            case 7: // Tux
                // für Guess, Farben
                arrGFXColor[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_empty.png"));
                arrGFXColor[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux.png"));
                arrGFXColor[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_mario.png"));
                arrGFXColor[3] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_xray.png"));
                arrGFXColor[4] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_space.png"));
                arrGFXColor[5] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_miraculix.png"));
                arrGFXColor[6] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_devil.png"));
                arrGFXColor[7] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_punk.png"));
                arrGFXColor[8] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_angry.png"));
                arrGFXColor[9] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_question.png"));
                arrGFXColor[10] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/tux_disabled.png"));
                // für Result
                arrGFXResult[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/empty_klein.png"));
                arrGFXResult[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/black.png"));
                arrGFXResult[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_7/white.png"));
                break;
            case 6: // World
                // für Guess, Farben
                arrGFXColor[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_empty.png"));
                arrGFXColor[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_yellow.png"));
                arrGFXColor[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_orange.png"));
                arrGFXColor[3] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_red.png"));
                arrGFXColor[4] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_violett.png"));
                arrGFXColor[5] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_blue.png"));
                arrGFXColor[6] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_turkis.png"));
                arrGFXColor[7] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_green.png"));
                arrGFXColor[8] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_brown.png"));
                arrGFXColor[9] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_question.png"));
                arrGFXColor[10] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_disabled.png"));
                // für Result
                arrGFXResult[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_empty_klein.png"));
                arrGFXResult[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_black.png"));
                arrGFXResult[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_6/world_white.png"));
                break;

            case 5: // Car
                // für Guess, Farben
                arrGFXColor[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_empty.png"));
                arrGFXColor[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_yellow.png"));
                arrGFXColor[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_orange.png"));
                arrGFXColor[3] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_red.png"));
                arrGFXColor[4] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_violett.png"));
                arrGFXColor[5] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_blue.png"));
                arrGFXColor[6] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_lightblue.png"));
                arrGFXColor[7] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_green.png"));
                arrGFXColor[8] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_grey.png"));
                arrGFXColor[9] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_question.png"));
                arrGFXColor[10] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/car_disabled.png"));
                // für Result
                arrGFXResult[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/empty_klein.png"));
                arrGFXResult[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/black.png"));
                arrGFXResult[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_5/white.png"));
                break;

            case 4: // Eulen
                // für Guess, Farben
                arrGFXColor[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_empty.png"));
                arrGFXColor[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl.png"));
                arrGFXColor[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_books.png"));
                arrGFXColor[3] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_bike.png"));
                arrGFXColor[4] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_ebook.png"));
                arrGFXColor[5] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_grill.png"));
                arrGFXColor[6] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_drunk.png"));
                arrGFXColor[7] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_notebook.png"));
                arrGFXColor[8] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_punk.png"));
                arrGFXColor[9] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_question.png"));
                arrGFXColor[10] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/owl_disabled.png"));
                // für Result
                arrGFXResult[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/empty_klein.png"));
                arrGFXResult[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/black.png"));
                arrGFXResult[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_4/white.png"));
                break;

            case 3: // Würfel
                // für Guess, Farben
                arrGFXColor[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_empty.png"));
                arrGFXColor[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_yellow.png"));
                arrGFXColor[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_orange.png"));
                arrGFXColor[3] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_red.png"));
                arrGFXColor[4] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_violett.png"));
                arrGFXColor[5] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_blue.png"));
                arrGFXColor[6] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_pink.png"));
                arrGFXColor[7] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_green.png"));
                arrGFXColor[8] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_brown.png"));
                arrGFXColor[9] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_question.png"));
                arrGFXColor[10] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_disabled.png"));
                // für Result
                arrGFXResult[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_empty_klein.png"));
                arrGFXResult[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_black.png"));
                arrGFXResult[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_3/cube_white.png"));
                break;

            case 2: // Smilies
                // für Guess, Farben
                arrGFXColor[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/empty.png"));
                arrGFXColor[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_yellow.png"));
                arrGFXColor[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_orange.png"));
                arrGFXColor[3] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_red.png"));
                arrGFXColor[4] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_violett.png"));
                arrGFXColor[5] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_blue.png"));
                arrGFXColor[6] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_pink.png"));
                arrGFXColor[7] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_green.png"));
                arrGFXColor[8] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_brown.png"));
                arrGFXColor[9] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/question.png"));
                arrGFXColor[10] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/smiley_disabled.png"));
                // für Result
                arrGFXResult[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/empty_klein.png"));
                arrGFXResult[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/black.png"));
                arrGFXResult[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_2/white.png"));
                break;

            default: // Farbe
                // für Guess, Farben
                arrGFXColor[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/empty.png"));
                arrGFXColor[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/yellow.png"));
                arrGFXColor[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/orange.png"));
                arrGFXColor[3] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/red.png"));
                arrGFXColor[4] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/violett.png"));
                arrGFXColor[5] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/blue.png"));
                arrGFXColor[6] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/turkis.png"));
                arrGFXColor[7] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/green.png"));
                arrGFXColor[8] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/brown.png"));
                arrGFXColor[9] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/question.png"));
                arrGFXColor[10] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/empty_disabled.png"));
                // für Result
                arrGFXResult[0] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/empty_klein.png"));
                arrGFXResult[1] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/black.png"));
                arrGFXResult[2] = new ImageIcon(getClass().getResource("/mastermind/graphix/design_1/white.png"));
                break;
        }
        // für Button

        icon_check = new ImageIcon(getClass().getResource("/mastermind/graphix/check.png"));
        icon_warning = new ImageIcon(getClass().getResource("/mastermind/graphix/warning.png"));
        icon_ok = new ImageIcon(getClass().getResource("/mastermind/graphix/ok.png"));
        icon_disabled = new ImageIcon(getClass().getResource("/mastermind/graphix/button_disabled.png"));
        icon_info = new ImageIcon(getClass().getResource("/mastermind/graphix/info.png"));
    }

    /**
     * init GUI
     */
    private void initGUI()
    {
        //System.out.println("initGUI...");
        // Title
        this.setTitle("Mastermind");

        // add custom icon
        this.setIconImage(new ImageIcon(getClass().getResource("/mastermind/graphix/mastermind.png")).getImage());

        // DefaultCloseOperator
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);

        // Listener initialisieren
        ml = new ClientGUIMouseListener(this);
        al = new ClientGUIActionListener(this);
        mwl = new ClientGUIMouseListener(this);

        // Panel
        gui_panel = new JPanel();
        // Create Menus and Items
        gui_menubar = new JMenuBar();
        this.setJMenuBar(gui_menubar);
        gui_menu_game = new JMenu("Game");
        gui_menu_cheat = new JMenu("Cheat");
        gui_menu_help = new JMenu("Help");
        gui_mi_new = new JMenuItem("New");
        gui_mi_new.addActionListener(al);
        gui_mi_newSolver = new JMenuItem("New Solver");
        gui_mi_newSolver.addActionListener(al);
        gui_mi_open = new JMenuItem("Open");
        gui_mi_open.addActionListener(al);
        gui_mi_save = new JMenuItem("Save");
        gui_mi_save.addActionListener(al);
        gui_mi_settings = new JMenuItem("Settings");
        gui_mi_settings.addActionListener(al);
        gui_mi_exit = new JMenuItem("Exit");
        gui_mi_exit.addActionListener(al);
        gui_mi_showHiddenPeg = new JMenuItem("Show one hidden Peg");
        gui_mi_showHiddenPeg.addActionListener(al);
        gui_mi_showHidden = new JMenuItem("Show all hidden Pegs");
        gui_mi_showHidden.addActionListener(al);

        gui_mi_help = new JMenuItem("Help");
        gui_mi_help.addActionListener(al);
        gui_mi_about = new JMenuItem("About");
        gui_mi_about.addActionListener(al);

        // Adding Menus and Items to Bar
        gui_menubar.add(gui_menu_game);
        gui_menubar.add(gui_menu_cheat);
        gui_menubar.add(gui_menu_help);

        gui_menu_game.add(gui_mi_new);
        gui_menu_game.add(gui_mi_newSolver);
        gui_menu_game.add(new JSeparator());
        gui_menu_game.add(gui_mi_open);
        gui_menu_game.add(gui_mi_save);
        gui_menu_game.add(new JSeparator());
        gui_menu_game.add(gui_mi_settings);
        gui_menu_game.add(new JSeparator());
        gui_menu_game.add(gui_mi_exit);

        gui_menu_cheat.add(gui_mi_showHiddenPeg);
        gui_menu_cheat.add(gui_mi_showHidden);
        gui_menu_help.add(gui_mi_help);
        gui_menu_help.add(gui_mi_about);

        // Array erstellen
        arrVal_guess = new int[nbrOfRows][nbrOfPegs];
        arrVal_result = new int[nbrOfRows][nbrOfPegs];
        arrVal_pattern = new int[nbrOfPegs];

        arrIcon_guess = new JLabel[nbrOfRows][nbrOfPegs];
        arrIcon_result = new JLabel[nbrOfRows][nbrOfPegs];
        arrIcon_pattern = new JLabel[nbrOfPegs];
        arrJPan = new JPanel[nbrOfRows];
        arrJButton = new JButton[nbrOfRows];

        gui_status = new JLabel();
        gui_status.setBounds(gapx_status_b, gapy_status_b, sizex_status, sizey_status);
        showTipp();

        // Positionen initialiseren
        // Panel & Buttons initialisieren
        for (int guessID = 0; guessID < nbrOfRows; guessID ++) {
            arrJPan[guessID] = new JPanel();
            arrJPan[guessID].setBorder(javax.swing.BorderFactory.createEtchedBorder());
            arrJPan[guessID].setBounds(gapx_panel_b, (guessID) * gapy_panel + gapy_panel_b, sizex_panel, sizey_panel);
            arrJPan[guessID].setLayout(null);
            arrJButton[guessID] = new JButton(icon_check);
            arrJButton[guessID].setDisabledIcon(icon_disabled);
            arrJButton[guessID].setBounds(gapx_button_b, gapy_button_b, sizex_button, sizey_button);
            if (guessID == nbrOfFilledRows) {
                arrJButton[guessID].setVisible(true);
            } else {
                arrJButton[guessID].setVisible(false);
            }
            arrJButton[guessID].addActionListener(al);
            arrJPan[guessID].add(arrJButton[guessID]);
        }

        // HiddenPattern initialisieren
        for (int pegID = 0; pegID < (nbrOfPegs); pegID ++) {
            // set color for Hidden Code
            arrVal_pattern[pegID] = ID_GFX_QUESTION;
            //normal peg
            arrIcon_pattern[pegID] = new JLabel(arrGFXColor[arrVal_pattern[pegID]]);
            arrIcon_pattern[pegID].setName("h" + pegID);
            arrIcon_pattern[pegID].addMouseListener(ml);
            arrIcon_pattern[pegID].addMouseWheelListener(mwl);
            arrIcon_pattern[pegID].setBounds((pegID * gapx_pattern) + gapx_pattern_b, gapy_pattern_b, sizex_pattern, sizey_pattern);
            arrIcon_pattern[pegID].setOpaque(true);
        }

        // Load History / Default Icon
        for (int guessID = 0; guessID < nbrOfRows; guessID ++) {
            // load history or init default
            if (guessID < nbrOfFilledRows) {
                // load history
                // set guess
                for (int pegID = 0; pegID < (nbrOfPegs); pegID ++) {
                    arrVal_guess[guessID][pegID] = mEngine.getHistoryGuess(guessID).getPeg(pegID);
                }
                // setButton
                disableButton(guessID);
                // set result
                int resultCount = 0;
                for (int peg = 0; peg < mEngine.getHistoryResult(guessID).getAnzPegColorPosRight(); peg ++) {
                    arrVal_result[guessID][peg] = ID_GFX_RESULT_BLACK; // schwarz
                    resultCount ++;
                }
                for (int peg = 0; peg < mEngine.getHistoryResult(guessID).getAnzPegColorRight(); peg ++) {
                    arrVal_result[guessID][resultCount] = ID_GFX_RESULT_WHITE; // weiss
                    resultCount ++;
                }
            } else { // set default
                // set guess
                for (int pegID = 0; pegID < (nbrOfPegs); pegID ++) {
                    arrVal_guess[guessID][pegID] = ID_GFX_EMPTY;
                }
                // set result: default empty
            }
        }

        // set color for history guesses
        for (int guessID = 0; guessID < nbrOfRows; guessID ++) {
            for (int pegID = 0; pegID < nbrOfPegs; pegID ++) {
                // Guess Label, Icon
                arrIcon_guess[guessID][pegID] = new JLabel(arrGFXColor[arrVal_guess[guessID][pegID]]);
                arrIcon_guess[guessID][pegID].setName(new Integer(guessID * 100 + pegID).toString());
                arrIcon_guess[guessID][pegID].setDisabledIcon(arrGFXColor[ID_GFX_EMPTY_DISABLED]);
                arrIcon_guess[guessID][pegID].setBounds((pegID * gapx_guess) + gapx_guess_b, gapy_guess_b, sizex_guess, sizey_guess);
                arrIcon_guess[guessID][pegID].setOpaque(true);
                if (guessID > nbrOfFilledRows) {
                    arrIcon_guess[guessID][pegID].setEnabled(false);
                }
                arrIcon_guess[guessID][pegID].addMouseListener(ml);
                arrIcon_guess[guessID][pegID].addMouseWheelListener(mwl);
                // Result Label, Icon
                arrIcon_result[guessID][pegID] = new JLabel(arrGFXResult[arrVal_result[guessID][pegID]]);
                if (pegID % 2 == 0) {
                    arrIcon_result[guessID][pegID].setName(new Integer(guessID * 100 + pegID).toString());
                    arrIcon_result[guessID][pegID].setBounds((pegID / 2 * gapx_result) + gapx_result_b, gapy_result_b + gapy_result_o1, sizex_result, sizey_result);
                    arrIcon_result[guessID][pegID].setOpaque(true);
                } else {
                    arrIcon_result[guessID][pegID].setName(new Integer(guessID * 100 + pegID).toString());
                    arrIcon_result[guessID][pegID].setBounds((((pegID + 1) / 2 - 1) * gapx_result) + gapx_result_b, gapy_result_b + gapy_result + gapy_result_o2, sizex_result, sizey_result);
                    arrIcon_result[guessID][pegID].setOpaque(true);
                }
            }
        }
        // Winner or not?
        if (nbrOfFilledRows >= 1) {
            if (mEngine.hasWon(getGuess(nbrOfFilledRows - 1))) {
                // Show Winner Message
                showHasWon();
                // disable Check Button & Pegs
                if (nbrOfFilledRows < nbrOfRows) {
                    for (int pegID = 0; pegID < nbrOfPegs; pegID ++) {
                        arrIcon_guess[nbrOfFilledRows][pegID].setEnabled(false);
                    }
                    hideButton(nbrOfFilledRows);
                }
            } else {
                if (nbrOfFilledRows >= nbrOfRows) {
                    showYouLoose();
                }
            }
        }

        // GUI mit Elementen füllen
        gui_panel.setLayout(null);
        for (int guessID = 0; guessID < nbrOfRows; guessID ++) {
            gui_panel.add(arrJPan[guessID]);
            for (int pegID = 0; pegID < nbrOfPegs; pegID ++) {
                arrJPan[guessID].add(arrIcon_guess[guessID][pegID]);
                arrJPan[guessID].add(arrIcon_result[guessID][pegID]);
            }
        }

        gui_panel.add(gui_status);
        for (int pegID = 0; pegID < nbrOfPegs; pegID ++) {
            gui_panel.add(arrIcon_pattern[pegID]);
        }

        this.setMinimumSize(new java.awt.Dimension(sizex_gui, sizey_gui));
        this.setResizable(false);
        // add(gui_grafix);
        this.add(gui_panel);

        this.pack();
        // in der mitte vom bildschirm anzeigen
        //this.setLocationRelativeTo(null);
    }

    /**
     * Startet neues Spiel, mit und ohne reset der Engine
     * @param resetEngine
     */
    public void startNewGame(boolean resetEngine)
    {
        if (resetEngine) {
            System.out.println("startNewGame...");
            mEngine.reset();
        } else {
            System.out.println("reloadGame...");
        }
        remove(gui_panel);
        doPlayMastermind();
    }

    /**
     * Startet neuen AutoSolver
     */
    @SuppressWarnings("static-access")
    public void startNewSolver()
    {
        JButton button_random = new JButton("Random");
        button_random.setBounds(20, 75, 85, 30);
        button_random.addActionListener(al);

        JButton button_ok = new JButton("OK");
        button_ok.setBounds(125, 75, 55, 30);

        // clear History
        mEngine.reset();
        hideHiddenPattern();
        this.repaint();

        myDialog = new JDialog(this, "Set Hidden Pattern", true);
        myDialog.setLayout(null);

        for (int pegID = 0; pegID < nbrOfPegs; pegID ++) {
            arrVal_pattern[pegID] = -1;
            myDialog.add(arrIcon_pattern[pegID]);
        }
        chg_hidden = true;
        gui_solver_status = new JLabel("Please set a valid combination.");
        gui_solver_status.setBounds(20, 55, 250, 15);
        myDialog.add(gui_solver_status);
        myDialog.add(button_ok);
        myDialog.add(button_random);

        button_ok.addActionListener(new java.awt.event.ActionListener()
        {

            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                if (GuessHandler.validateGuess(new Guess(arrVal_pattern))) {
                    myDialog.dispose();
                }
            }
        });
        myDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        myDialog.setSize(((nbrOfPegs) * gapx_pattern) + 2 * gapx_pattern_b, 150);
        myDialog.setResizable(false);
        myDialog.setLocationRelativeTo(this);
        myDialog.setVisible(true);

        showSolving();
        this.repaint();
        System.out.println("Solver, try to find this code:");
        mEngine.setHiddenPattern(new Guess(arrVal_pattern), true);

        long startTime, endTime;
        startTime = System.currentTimeMillis();

        //try to solve...
        Guess guess = new Guess();
        Result result = null;
        // Get Settings
        int numberOfPegs = SettingsHandler.getInstance().getSettings().getNumberOfPegs();

        // First Guess
        if (mEngine.getNumberOfGuesses() == 0) {
            // Calc First guess
            if (SettingsHandler.getInstance().getSettings().getAllowDuplicateColor()) {
                // doppelte farben
                int[] arrColor = new int[numberOfPegs];
                for (int i = 0; i < numberOfPegs; i ++) {
                    arrColor[i] = (int) (Math.round(i / (double) numberOfPegs)) + 1;
                }
                guess = new Guess(arrColor);
            } else {
                // keine doppelte farben
                int[] arrColor = new int[numberOfPegs];
                for (int i = 0; i < numberOfPegs; i ++) {
                    arrColor[i] = i + 1;
                }
                guess = new Guess(arrColor);
            }
            // Check Result, maybe we have won
            System.out.println("1: " + guess);
            // Get Result (used for next guess calc)
            result = mEngine.getResult(guess);
        }

        // 2nd und weitere Versuche...
        if ( ! mEngine.hasWon(guess)) {
            SolverLogik solver = new SolverLogik(guess, result);
            // try until we won or until max rows
            for (int i = 2; i <= SettingsHandler.getInstance().getSettings().getNumberOfRows(); i ++) {
                // get a Guess from solver
                guess = solver.getNextGuess(result, i);
                if (guess == null) {
                    break;
                }
                // Show Guess
                System.out.println(i + ": " + guess);
                // Get Result (used for next guess calc)
                result = mEngine.getResult(guess);
                if (mEngine.hasWon(guess)) {
                    break;
                }
            }
        }

        endTime = System.currentTimeMillis();
        Long lngExecTime = endTime - startTime;
        String strExecTime = String.format("%d min, %d sec",
            TimeUnit.MILLISECONDS.toMinutes(lngExecTime),
            TimeUnit.MILLISECONDS.toSeconds(lngExecTime) -
            TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(lngExecTime)));
        System.out.println("Exec: " + strExecTime);

        // wait till close....
        //System.out.println("Show Settings Dialog - finish...");
        chg_hidden = false;
        remove(gui_panel);
        setCursor(Cursor.getDefaultCursor());
        doPlayMastermind();
        showHiddenPattern();
    }

    /**
     * inkrementiert Peg mit der enstprechenden PegID und GuessID
     * @param guessID Versuch Nr
     * @param pegID Peg Nr
     */
    public void incValue(int guessID, int pegID)
    {
        if (nbrOfFilledRows == guessID) {
            if (arrVal_guess[guessID][pegID] == nbrOfIcons) {
                arrVal_guess[guessID][pegID] = 1;
            } else {
                arrVal_guess[guessID][pegID] = (arrVal_guess[guessID][pegID] + 1);
            }
        }
        arrIcon_guess[guessID][pegID].setIcon(arrGFXColor[arrVal_guess[guessID][pegID]]);
    }

    /**
     * dekrementiert Peg mit der enstprechenden PegID und GuessID
     * @param guessID Versuch Nr
     * @param pegID Peg Nr
     */
    public void decValue(int guessID, int pegID)
    {
        if (nbrOfFilledRows == guessID) {
            if (arrVal_guess[guessID][pegID] <= 1) {
                arrVal_guess[guessID][pegID] = nbrOfIcons;
            } else {
                arrVal_guess[guessID][pegID] = (arrVal_guess[guessID][pegID] - 1);
            }
        }
        arrIcon_guess[guessID][pegID].setIcon(arrGFXColor[arrVal_guess[guessID][pegID]]);
    }

    /**
     * inkrementiert HiddenPeg mit der enstprechenden PegID
     * @param pegID
     */
    public void incHiddenValue(int pegID)
    {
        if (chg_hidden == true) {
            if ((arrVal_pattern[pegID] < 1) || (arrVal_pattern[pegID] >= nbrOfIcons)) {
                arrVal_pattern[pegID] = 1;
            } else {
                arrVal_pattern[pegID] += 1;
            }
        }
        arrIcon_pattern[pegID].setIcon(arrGFXColor[arrVal_pattern[pegID]]);
    }

    /**
     * dekrementiert HiddenPeg mit der enstprechenden PegID
     * @param pegID
     */
    public void decHiddenValue(int pegID)
    {
        if (chg_hidden == true) {
            if (arrVal_pattern[pegID] <= 1) {
                arrVal_pattern[pegID] = nbrOfIcons;
            } else {
                arrVal_pattern[pegID] -= 1;
            }
        }

        arrIcon_pattern[pegID].setIcon(arrGFXColor[arrVal_pattern[pegID]]);
    }

    /**
     * deaktiviert Button mit der entsprechenden ButtonID
     * @param buttonID
     */
    public void disableButton(int buttonID)
    {
        arrJButton[buttonID].setVisible(true);
        arrJButton[buttonID].setEnabled(false);
        arrJButton[buttonID].setBorder(null);
    }

    /**
     * versteckt check Button mit der entsprechenden ButtonID
     * @param buttonID
     */
    public void hideButton(int buttonID)
    {
        arrJButton[buttonID].setVisible(false);
    }

    /**
     * Wechselt den zuständigen check-Button
     */
    public void chg_button()
    {
        // attribute initialisieren
        boolean validSettings;
        boolean won;
        Result newGuessResult;
        Guess newGuess;
        // der zu überprüfende guess holen
        newGuess = getGuess(nbrOfFilledRows);
        System.out.println("Check Guess: " + newGuess);
        // Rateversuch zulässig?
        validSettings = GuessHandler.validateGuess(newGuess);
        if ( ! validSettings) {
            System.out.println("no valid guess...");
            showInvalidGuess();

        } else {
            newGuessResult = mEngine.getResult(newGuess);
            won = mEngine.hasWon(newGuess);
            System.out.println("Valid Guess! Result: " + newGuessResult + ", Winner: " + won);
            setResult(newGuessResult, nbrOfFilledRows);
            nbrOfFilledRows ++;
            if (won) {
                disableButton(nbrOfFilledRows - 1);
                showHiddenPattern();
                showHasWon();
            } else if (nbrOfFilledRows < nbrOfRows) {
                disableButton(nbrOfFilledRows - 1);
                arrJButton[nbrOfFilledRows].setVisible(true);
                arrJButton[nbrOfFilledRows].requestFocus();
                showNothing();
                // enable next row
                for (int pegID = 0; pegID < nbrOfPegs; pegID ++) {
                    arrIcon_guess[nbrOfFilledRows][pegID].setEnabled(true);
                }
            } else {
                disableButton(nbrOfFilledRows - 1);
                showHiddenPattern();
                showYouLoose();
            }
        }
    }

    private Guess getGuess(int row)
    {
        int[] atemp = new int[SettingsHandler.getInstance().getSettings().getNumberOfPegs()];
        for (int i = 0; i < SettingsHandler.getInstance().getSettings().getNumberOfPegs(); i ++) {
            atemp[i] = arrVal_guess[row][i];
        }
        Guess temp = new Guess(atemp);
        return temp;
    }

    private void setResult(Result newResult, int guessID)
    {
        int resultCount = 0;
        for (int peg = 0; peg < newResult.getAnzPegColorPosRight(); peg ++) {
            arrVal_result[guessID][peg] = ID_GFX_RESULT_BLACK; // schwarz
            resultCount ++;
        }
        for (int peg = 0; peg < newResult.getAnzPegColorRight(); peg ++) {
            arrVal_result[guessID][resultCount] = ID_GFX_RESULT_WHITE; // weiss
            resultCount ++;
        }
        for (int peg = 0; peg < resultCount; peg ++) {
            arrIcon_result[guessID][peg].setIcon(arrGFXResult[arrVal_result[guessID][peg]]);
        }
    }

    /**
     * Shows one hidden Peg (Random)
     */
    public void showHiddenPeg()
    {
        int nbrOfHiddenPegs = 0;
        int[] posHiddenPeg = new int[nbrOfPegs];
        //count number of hidden pegs
        for (int pegID = 0; pegID < (nbrOfPegs); pegID ++) {
            if (arrVal_pattern[pegID] == ID_GFX_QUESTION) {
                posHiddenPeg[nbrOfHiddenPegs] = pegID;
                nbrOfHiddenPegs ++;
            }
        }
        if (nbrOfHiddenPegs > 0) {
            if (nbrOfHiddenPegs > 1) {
                // random
                Random rnd = new Random();
                int pos = (rnd.nextInt(nbrOfHiddenPegs));
                showHiddenPeg(posHiddenPeg[pos]);
            } else {
                // only one peg left to show
                showHiddenPeg(posHiddenPeg[0]);
            }
        }
    }

    /**
     * Zeigt ein hidden Peg an
     * @param pegID Position die angezeigt werden soll
     */
    public void showHiddenPeg(int pegID)
    {
        if (pegID >= 0 && pegID <= nbrOfPegs) {
            arrVal_pattern[pegID] = mEngine.getHiddenPattern().getPeg(pegID);
            arrIcon_pattern[pegID].setIcon(arrGFXColor[arrVal_pattern[pegID]]);
        }
    }

    /**
     * Zeigt HiddenPattern an
     */
    public void showHiddenPattern()
    {
        for (int i = 0; i < (nbrOfPegs); i ++) {
            showHiddenPeg(i);
        }
    }

    /*
     * versteckt HiddenPattern
     */
    private void hideHiddenPattern()
    {
        for (int i = 0; i < (nbrOfPegs); i ++) {
            arrVal_pattern[i] = mEngine.getHiddenPattern().getPeg(i);
            arrIcon_pattern[i].setIcon(arrGFXColor[ID_GFX_QUESTION]);
        }
    }

    /**
     * save Engine to file
     * @param file name of file
     */
    public void saveGame(File file)
    {
        // save Engine to file
        mEngine.saveEngine(file);
    }

    /**
     * load Engine from file an reset game
     * @param file file to load
     */
    protected void loadGame(File file)
    {
        // load Engine from file
        mEngine.loadEngine(file);

        // remove old panel
        remove(gui_panel);

        // Init & Play
        doPlayMastermind();

        if (mEngine.getNumberOfGuesses() >= nbrOfRows) {
            // game end
            showHiddenPattern();
        }

    }

    /*
     * Zeigt Meldung bei gewonnenem Spiel an und zeigt verstecktes Muster an
     */
    private void showHasWon()
    {
        System.out.println("Congratulations!");
        gui_status.setText("Congratulations!");
        gui_status.setIcon(icon_ok);
        showHiddenPattern();
    }

    /*
     * Zeigt Meldung bei einem verlorenen Spiel und zeigt verstecktes Pattern an
     */
    private void showYouLoose()
    {
        System.out.println("Sorry, You loose!");
        // Meldung mit Icon ausgeben
        gui_status.setText("Sorry, You loose!");
        gui_status.setIcon(null);
        // HiddenPattern anzeigen
        showHiddenPattern();
    }

    /*
     * Zeigt Meldung im Status an, dass Kombination invalid ist
     */
    private void showInvalidGuess()
    {
        // Meldung mit Icon ausgeben
        gui_status.setText("Please enter a valid combination");
        gui_status.setIcon(icon_warning);
    }

    /*
     * Löscht Statuszeile
     */
    private void showNothing()
    {
        // Meldung mit Icon ausgeben
        gui_status.setText(null);
        gui_status.setIcon(null);
    }

    /*
     * Zeigt Tipp in Status an
     */
    private void showTipp()
    {
        gui_status.setText("Tipp: Use Mousewheel to change Peg");
        gui_status.setIcon(icon_info);
    }

    /**
     * Validate new hidden Guess
     */
    public void validateNewHiddenGuess()
    {
        if (GuessHandler.validateGuess(new Guess(arrVal_pattern))) {
            gui_solver_status.setText("");
        } else {
            gui_solver_status.setText("Please set a valid combination.");
        }
    }

    /**
     * generates Random Hidden Pattern for Solver
     */
    protected void generateRandom()
    {
        // Random Pattern generieren
        mEngine.setHiddenPattern(RandomGuess.genRandomGuess(), false);
        // Hidden Pattern anzeigen
        showHiddenPattern();
    }

    /**
     * show Solving in Progress and change Cursor
     */
    private void showSolving()
    {
        gui_status.setText("Solving in progress... please wait");
        gui_status.setIcon(icon_info);
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    }
}
