package mastermind.logic;

import java.util.Hashtable;
import java.util.Vector;

/**
 * Mastermind CMD Argument Parser
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-05.v01   FeRo    init version
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-05.v01
 */
public class ArgParser
{

    private Vector<String> params = new Vector<String>();
    private Hashtable<String,String> options = new Hashtable<String,String>();
    private int paramIndex = 0;
    private int numberOfParams = 0;

    /**
     * Konstruktor
     * @param args CMD-Params
     */
    public ArgParser(String[] args)
    {
        String key;
        String value;
        for (int i = 0; i < args.length; i ++) {
            if (args[i].startsWith("-") || args[i].startsWith("/")) {
                numberOfParams++;
                int pos = args[i].indexOf("=");
                if (pos > 0) {
                    key = args[i].substring(1, pos);
                    value = args[i].substring(pos + 1);
                } else {
                    key = args[i].substring(1);
                    value = "";
                }
                options.put(key.toLowerCase(), value);
            } else {
                params.addElement(args[i]);
            }
        }
    }

    /**
     * Gibt es diesen Parameter
     * @param opt Name des Parameters
     * @return true falls vorhanden, sonst false
     */
    public boolean hasOption(String opt)
    {
        return options.containsKey(opt.toLowerCase());
    }

    /**
     * Gibt den Parameter, Value zurück
     * @param opt Name des Parameters
     * @return Value des Parameters
     */
    public String getOption(String opt)
    {
        return (String) options.get(opt.toLowerCase());
    }

    /**
     * Gibt den nachfolgenden Parameter zurück
     * @return String des Parameters
     */
    public String nextParam()
    {
        if (paramIndex < params.size()) {
            return (String) params.elementAt(paramIndex ++);
        }
        return null;
    }

    /**
     * Gibt die Anzahl cmd Params zurück
     * @return Anzahl Parameter
     */public int numberOfParams() {
        return numberOfParams;
    }
}
