package mastermind.solver;

import java.util.List;
import mastermind.model.Guess;
import mastermind.model.Result;

/**
 * Mastermind Solver: Thread Class für NextGuess berechnen
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-25.v01   FeRo    init version
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-25.v01
 */
public class SolverThreadGetNextGuess implements Runnable
{

    private int threadNum;
    private List<Guess> partPossibleGuess;
    private Guess lastGuess;
    private Result lastResult;
    private SolverLogik logik;

    /**
     * Default Konstruktor
     * @param logik SolverLogik (Parent)
     * @param lastGuess Letzter Guess
     * @param lastResult Letztes Resultat
     * @param id ID des Threads
     */
    public SolverThreadGetNextGuess(SolverLogik logik, Guess lastGuess, Result lastResult, int id)
    {
        this.logik = logik;
        this.lastGuess = lastGuess;
        this.lastResult = lastResult;
        threadNum = id;
    }

    /**
     * Thread run() - eigentlicher Code
     */
    public void run()
    {
        //System.out.println(" >> " + threadNum + " (in)  Elem: " + partPossibleGuess.size());
        for (int i = partPossibleGuess.size() - 1; i >= 0; i --) {
            Result result = this.lastGuess.compareGuess(partPossibleGuess.get(i));
//                System.out.println("  > Last Guess: " + lastGuess);
//                System.out.println("        Result: " + lastResult);
//                System.out.println("  > Poss.Guess: " + possibleGuess.get(i));
//                System.out.println("        Result: " + result);
            if ( ! result.equals(lastResult)) {
                partPossibleGuess.remove(i);
            }
        }
        //System.out.println(" >> " + threadNum + " (out) Elem: " + partPossibleGuess.size());

        // save it back
        if (partPossibleGuess.size() > 0) {
            logik.setListGuessResult(partPossibleGuess);
        }
    }

    /**
     * Setter Methode um die zu überprüfende Liste von Guess zu übergeben
     * @param myList Liste mit möglichen Guess
     */
    public void setGuessList(List<Guess> myList)
    {
        this.partPossibleGuess = myList;
    }
}
