package mastermind.engine;

import java.util.logging.Level;
import java.util.logging.Logger;
import mastermind.logic.GuessHandlerTest;
import mastermind.logic.SettingsHandler;
import mastermind.model.Guess;
import mastermind.model.Result;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public class MastermindEngineTest
{

    public MastermindEngineTest()
    {
    }

    /**
     * Test of hasWon method, of class MastermindEngine.
     */
    @Test
    public void testHasWon_True()
    {
        System.out.println("hasWon: True");
        MastermindEngine instance = new MastermindEngine();
        // define settings
        int numberOfColors = 4;
        boolean allowDuplicateColor = false;
        int numberOfPegs = 4;
        int numberOfRows = 4;
        try {
            SettingsHandler.getInstance().getSettings().setSettings(numberOfColors, allowDuplicateColor, numberOfPegs, numberOfRows);
        } catch (Exception ex) {
            Logger.getLogger(GuessHandlerTest.class.getName()).log(Level.SEVERE, null, ex);
        }
        // set hidden pattern
        Guess hiddenPattern = new Guess(new int[]{1, 2, 3, 4});
        instance.setHiddenPattern(hiddenPattern, false);
        // guess
        Guess guess = new Guess(new int[]{1, 2, 3, 4});
        boolean expResult = true;
        boolean result = instance.hasWon(guess);
        assertEquals(expResult, result);
    }

    /**
     * Test of hasWon method, of class MastermindEngine.
     */
    @Test
    public void testHasWon_False()
    {
        System.out.println("hasWon: False");
        MastermindEngine instance = new MastermindEngine();
        // define settings
        int numberOfColors = 4;
        boolean allowDuplicateColor = false;
        int numberOfPegs = 4;
        int numberOfRows = 4;
        try {
            SettingsHandler.getInstance().getSettings().setSettings(numberOfColors, allowDuplicateColor, numberOfPegs, numberOfRows);
        } catch (Exception ex) {
            Logger.getLogger(GuessHandlerTest.class.getName()).log(Level.SEVERE, null, ex);
        }
        // set hidden pattern
        Guess hiddenPattern = new Guess(new int[]{1, 2, 3, 4});
        instance.setHiddenPattern(hiddenPattern, false);
        // guess
        Guess guess = new Guess(new int[]{4, 3, 2, 1});
        boolean expResult = false;
        boolean result = instance.hasWon(guess);
        assertEquals(expResult, result);
    }

    /**
     * Test of getResult method, of class MastermindEngine.
     */
    @Test
    public void testGetResult()
    {
        System.out.println("getResult");
        
        MastermindEngine instance = new MastermindEngine();
        // define settings
        int numberOfColors = 4;
        boolean allowDuplicateColor = true;
        int numberOfPegs = 4;
        int numberOfRows = 4;
        try {
            SettingsHandler.getInstance().getSettings().setSettings(numberOfColors, allowDuplicateColor, numberOfPegs, numberOfRows);
        } catch (Exception ex) {
            Logger.getLogger(GuessHandlerTest.class.getName()).log(Level.SEVERE, null, ex);
        }
        // set hidden pattern
        Guess hiddenPattern = new Guess(new int[]{1, 2, 3, 4});
        instance.setHiddenPattern(hiddenPattern, false);

        // guess
        Guess guess = new Guess(new int[]{1, 2, 3, 3});
        Result expResult = new Result(3,0);
        
        Result result = instance.getResult(guess);
        assertTrue(expResult.toString().equals(result.toString()));
    }

    /**
     * Test of getHiddenPattern method, of class MastermindEngine.
     */
    @Test
    public void testGetHiddenPattern()
    {
        System.out.println("setHiddenPattern / getHiddenPattern");
        MastermindEngine instance = new MastermindEngine();
        // define settings
        int numberOfColors = 4;
        boolean allowDuplicateColor = true;
        int numberOfPegs = 4;
        int numberOfRows = 4;
        try {
            SettingsHandler.getInstance().getSettings().setSettings(numberOfColors, allowDuplicateColor, numberOfPegs, numberOfRows);
        } catch (Exception ex) {
            Logger.getLogger(GuessHandlerTest.class.getName()).log(Level.SEVERE, null, ex);
        }
        // set hidden pattern
        Guess hiddenPattern = new Guess(new int[]{1, 2, 3, 4});
        instance.setHiddenPattern(hiddenPattern, false);

        Guess expResult = new Guess(new int[]{1, 2, 3, 4});
        Guess result = instance.getHiddenPattern();
        assertTrue(expResult.equals(result));
    }
}
